#include "DeDeEncCarBase_V1_0.h"

// WS2812控制
MyNeoPixel::MyNeoPixel(uint16_t numLeds, uint8_t pin)
: strip(numLeds, pin, NEO_GRB + NEO_KHZ800) {}

void MyNeoPixel::begin() {
  strip.begin();
  strip.show();  // 初始化后，清空任何之前的数据
}

void MyNeoPixel::colorWipe(uint32_t color) {
  for (int i = 0; i < strip.numPixels(); i++) {  // 依次为每个LED设置颜色
    strip.setPixelColor(i, color);
    strip.show();
  }
}

void MyNeoPixel::red() {
  colorWipe(strip.Color(255, 0, 0));
}

void MyNeoPixel::white() {
  colorWipe(strip.Color(255, 255, 255));
}

void MyNeoPixel::green_blue() {
  colorWipe(strip.Color(0, 255, 255));
}

void MyNeoPixel::green() {
  colorWipe(strip.Color(0, 255, 0));
}

void MyNeoPixel::blue() {
  colorWipe(strip.Color(0, 0, 255));
}

void MyNeoPixel::turnOff() {
  colorWipe(strip.Color(0, 0, 0));
}

void MyNeoPixel::light() {
  colorWipe(strip.Color(255, 255, 255));
}

SimpleServoControl::SimpleServoControl(SoftwareSerial& serialRef)
  : serial(&serialRef), cur_angle(0) {
  int servo_sdataInit[10] = { 0x7b, 0x79, 0, 0, 0, 0, 0x10, 0x10, 0x10, 0x7d };
  memcpy(servo_sdata, servo_sdataInit, sizeof(servo_sdataInit));
}

void SimpleServoControl::senddata() {
  for (int i = 0; i < 10; i++) {
    serial->write(servo_sdata[i]);
  }
}

void SimpleServoControl::set_angle(int id_num, float angle, int steps) {
  // 省略具体实现，与原代码一致
  if (steps <= 0) steps = 1;  // 确保步数为正数
    if (angle < 0) angle = 0;
    if (angle > 275) angle = 275;

    // 设置舵机ID和目标角度
    servo_sdata[0] = 123;
    servo_sdata[1] = id_num;
    servo_sdata[2] = (int)((int)(angle / 10)) % 100;
    servo_sdata[3] = ((int)(angle * 10)) % 100;
    servo_sdata[4] = (int)(steps / 100);
    servo_sdata[5] = (int)((int)steps % 100);
    servo_sdata[7] = 16;
    servo_sdata[8] = 1;
    servo_sdata[9] = 125;

    servo_sdata[6] = (servo_sdata[1] + servo_sdata[2] + servo_sdata[3] + servo_sdata[4] + servo_sdata[5] + servo_sdata[7] + servo_sdata[8]) % 100;
    senddata();  // 发送数据
    delay(2);    // 等待舵机响应
    servo_sdata[7] = 17;
    servo_sdata[6] = (servo_sdata[1] + servo_sdata[2] + servo_sdata[3] + servo_sdata[4] + servo_sdata[5] + servo_sdata[7] + servo_sdata[8]) % 100;
    // 串口发送
    senddata();  // 发送数据
    delay(2);    // 等待舵机响应
}

float SimpleServoControl::get_state(int id_num, int para_num, int o_m) {
  // 省略具体实现，与原代码一致
  if (id_num == 121 && o_m == 0) {
      return 0;
    } else if (id_num == 121 && o_m > 1) {
      return 0;
    } else {
      servo_sdata[0] = 123;
      servo_sdata[1] = id_num;
      servo_sdata[7] = 0x13;
      servo_sdata[9] = 125;
      servo_sdata[6] = (servo_sdata[1] + servo_sdata[2] + servo_sdata[3] + servo_sdata[4] + servo_sdata[5] + servo_sdata[7] + servo_sdata[8]) % 100;

      u_numm = 0;
      senddata();
      char i = 0;
      int t = 1000;
      bool n_s = 1;
      char num = 16;
      while (serial->available() < num && t > 0) {
        t -= 1;
        if (serial->available() > 0 && n_s) {
          if (serial->read() == 123) {
            n_s = 0;
            servo_recvdata[0] = 123;
            u_numm = 1;
          }
        }
      }
      while (serial->available() > 0) {
        servo_recvdata[u_numm] = (int)serial->read();
        u_numm++;
      }
      if (u_numm == num) {
        id_number = servo_recvdata[1];
        cur_angle = (double)(servo_recvdata[2] * 10 + servo_recvdata[3] * 0.1);
        exp_angle = servo_recvdata[4] * 10 + servo_recvdata[5] * 0.1;
        run_time = servo_recvdata[6] * 100 + servo_recvdata[7];
        servo_rpara[0] = id_number;
        servo_rpara[1] = cur_angle;
        servo_rpara[2] = exp_angle;
        servo_rpara[3] = run_time;
        if (para_num == 1)
          return id_number;
        else if (para_num == 2)
          return cur_angle;
        else if (para_num == 3)
          return exp_angle;
        else if (para_num == 4)
          return run_time;
        else if (para_num == 0) {
          cur_mode = servo_recvdata[8];
          servo_rpara[4] = cur_mode;
        }
      } else return -1;  //返回数据失败  可做读取失败标志位
    }
}

// cmd文件
bool GetPosiState(uint8_t address, uint8_t motorID) {
  uint8_t* msg = readI2CMsgHaveGotTargetPosi(address, motorID);
  Wire.beginTransmission(address);
  Wire.write(msg, 12);
  Wire.endTransmission();
  delay(2);

  int get_state = 0;
  byte RxData[8];
  int index = 0;

  unsigned long startMillis = millis();  // 开始时间
  unsigned long maxWait = 2000;          // 设置最大等待时间为2000毫秒

  Wire.requestFrom(address, 8);
  while (Wire.available() && index < 8) {
    if (millis() - startMillis > maxWait)  // 判断是否超过最大等待时间
    {
      Serial.println("Get Return Error");
      return false;  // 超时则返回false
    }
    RxData[index] = Wire.read();
    index++;
  }

  if (index == 8) {
    byte checkSum = computeXOR(RxData, 7);

    if (checkSum != RxData[7]) {
      return false;
    }

    get_state = (RxData[1] << 8) | RxData[2];

    if (get_state == 1) {
      return true;
    } else {
      return false;
    }
  }
}

// 17、读取判断是否到达目标距离位置
uint8_t* readI2CMsgChassisHaveGotTargetPosi(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = CHASSIS_READ_HAVE_GOT_POSI;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

bool GetChassisPosiState(uint8_t address, uint8_t motorID) {
  uint8_t* msg = readI2CMsgChassisHaveGotTargetPosi(address, motorID);
  Wire.beginTransmission(address);
  Wire.write(msg, 12);
  Wire.endTransmission();
  delay(2);

  int get_state = 0;
  byte RxData[8];
  int index = 0;

  unsigned long startMillis = millis();  // 开始时间
  unsigned long maxWait = 2000;          // 设置最大等待时间为2000毫秒

  Wire.requestFrom(address, 8);
  while (Wire.available() && index < 8) {
    if (millis() - startMillis > maxWait)  // 判断是否超过最大等待时间
    {
      Serial.println("Get Return Error");
      return false;  // 超时则返回false
    }
    RxData[index] = Wire.read();
    index++;
  }

  if (index == 8) {
    byte checkSum = computeXOR(RxData, 7);

    if (checkSum != RxData[7]) {
      return false;
    }

    get_state = (RxData[1] << 8) | RxData[2];

    if (get_state == 1) {
      return true;
    } else {
      return false;
    }
  }
}

uint8_t* createChassisPIDMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int Kp, int Ki, int Kd) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // 通用的标识
  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  // PID参数的Kp设定
  cmd[3] = (abs(Kp) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(Kp) & 0xFF;         // Low byte

  // PID参数的Ki设定
  cmd[5] = (abs(Ki) >> 8) & 0xFF;  // High byte
  cmd[6] = abs(Ki) & 0xFF;         // Low byte

  // PID参数的Kd设定
  cmd[7] = (abs(Kd) >> 8) & 0xFF;  // High byte
  cmd[8] = abs(Kd) & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  return cmd;
}

void InitChassisStraightLineAnglePIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    // Serial.println("Send Speed PID Setting");
    uint8_t* msg = createChassisPIDMsg(address, chassis, chassis_cmd, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数


    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    // Serial.println("STOP");           // 打印空格，方便查看每个数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    msg = readI2CMsgChassisPID(address, chassis, CHASSIS_STRAIGHT_LINE_ANGLE_READ);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    // Serial.println("Get Speed PID Setting");
    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);
    // Serial.println(p_get);
    // Serial.println(i_get);
    // Serial.println(d_get);
    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Chassis Straight Line Angle Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        delay(1000);
      }
    }
  }
}

// PositionCorrectionPID
void InitChassisPositionCorrectionPIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    // Serial.println("Send Speed PID Setting");
    uint8_t* msg = createChassisPIDMsg(address, chassis, chassis_cmd, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数


    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    msg = readI2CMsgChassisPID(address, chassis, CHASSIS_Position_Correction_READ);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    // Serial.println("Get Speed PID Setting");
    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);
    // Serial.println(p_get);
    // Serial.println(i_get);
    // Serial.println(d_get);
    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Position Correction PID Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        delay(1000);
      }
    }
  }
}

// TurnAngleLockPID
void InitChassisTurnAngleLockPIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    // Serial.println("Send Speed PID Setting");
    uint8_t* msg = createChassisPIDMsg(address, chassis, chassis_cmd, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数


    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    msg = readI2CMsgChassisPID(address, chassis, CHASSIS_Turn_Angle_Lock_READ);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    // Serial.println("Get Speed PID Setting");
    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);
    // Serial.println(p_get);
    // Serial.println(i_get);
    // Serial.println(d_get);
    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Turn Angle Lock PID Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        delay(1000);
      }
    }
  }
}

// 4、读取速度控制的PID参数
uint8_t* readI2CMsgChassisPID(uint8_t address, uint8_t chassis, uint8_t cmd_chassis) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = cmd_chassis;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

void InitPosiPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    uint8_t* msg = createPosiPIDMsg(address, motorID, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数
    Wire.beginTransmission(address);                                                     // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);                                                                 // 发送3个字节的数据
    Wire.endTransmission();                                                              // 关闭通信
    delay(10);                                                                           // 短暂延迟

    msg = readI2CMsgPosiPID(address, motorID);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);

    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Posi PID Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        //        Serial.println("Posi PID Setting Error.");
        delay(1000);
      }
    }
  }
}

void ChassisStop(uint8_t address, uint8_t chassis, uint8_t chassis_cmd) {
  for (int retry = 0; retry < MAX_RETRY_TIMES; ++retry) {
    uint8_t* msg = createChassisStopMsg(address, chassis, chassis_cmd);  // 设置速度PID参数

    uint8_t msg_check = msg[11];

    // 发送控制底盘的指令
    Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);                 // 发送3个字节的数据
    Serial.println("Send ChassisTurnAngleWoLock cmd");  // 打印空格，方便查看每个数据
    Wire.endTransmission();              // 关闭通信

    // 要加入到达之后的信号 
    // 要加入验证、重发的策略

    // delay(5);
    // 发送完成后需要再校验下
    msg = readI2CMsgCmd(address);
    Wire.beginTransmission(address);
    Wire.write(msg, 12);
    Wire.endTransmission();

    byte rece_cmd = 99;
    GetCMDPara(address, rece_cmd);

    if (rece_cmd == msg_check) {
      Serial.println("Stop Success!");
      break;
    } else {
      Serial.println(rece_cmd);
      Serial.println(msg_check);
    }
  }
}

// 底盘控制代码
uint8_t* createChassisStopMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd) {
  static uint8_t cmd[12];

  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  // 假设 turn_state 直接以整数形式发送
  cmd[3] = 0;  // turn_state 高字节
  cmd[4] = 0;         // turn_state 低字节

  cmd[5] = 0;  // v_angle 高字节
  cmd[6] = 0;         // v_angle 低字节

  cmd[7] = 0;  // w_speed 高字节
  cmd[8] = 0;         // w_speed 低字节

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // 计算校验和
  cmd[11] = computeXOR(cmd, 11);  // 只计算前9个字节的XOR

  return cmd;
}

void ChassisGoV(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float w_speed) {
  uint8_t* msg = createVChassisCtrlMsg(address, chassis, chassis_cmd, v_speed, v_angle, 0, w_speed);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send chassis cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信
}

void ChassisGoVAngleLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float angle_lock) {
  uint8_t* msg = createVChassisCtrlMsg(address, chassis, chassis_cmd, v_speed, v_angle, angle_lock, 0);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send chassis cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信
}

void ChassisGoDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float angle, float angle_lock, int distance, int speed_rate, int init_speed) {
  uint8_t* msg = createChassisGoDistanceMsg(address, chassis, chassis_cmd, angle, angle_lock, distance, speed_rate, init_speed);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send chassis cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信

  // 发送完成
  // 要去获取此时是否到指定的位置
  volatile int chassis_get_state = 0;


  while ((chassis_get_state) < 1) {
    chassis_get_state = GetChassisPosiState(address, chassis);
  }
}




void InitSpeedPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    // Serial.println("Send Speed PID Setting");
    uint8_t* msg = createSpeedPIDMsg(address, motorID, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数

    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Serial.println("STOP");           // 打印空格，方便查看每个数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    // while(1);

    // Serial.println("Send Get Speed PID Setting");
    msg = readI2CMsgSpeedPID(address, motorID);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    // Serial.println("Get Speed PID Setting");
    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);
    Serial.println(p_get);
    Serial.println(i_get);
    Serial.println(d_get);
    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Speed PID Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        Serial.println("Speed PID Setting Error.");
        delay(1000);
      }
    }
  }
}

void resetI2Cbus(byte sdaPin, byte sclPin) {
  pinMode(sdaPin, OUTPUT);  // 将SDA脚设置为输出
  pinMode(sclPin, OUTPUT);  // 将SCL脚设置为输出

  digitalWrite(sclPin, HIGH);  // 保持SCL高电平
  delayMicroseconds(5);

  for (int i = 0; i < 10; i++) {  // 生成10个模拟的I2C时钟周期
    digitalWrite(sdaPin, HIGH);
    delayMicroseconds(5);
    digitalWrite(sclPin, LOW);
    delayMicroseconds(5);
    digitalWrite(sclPin, HIGH);
    delayMicroseconds(5);
  }

  Wire.begin();  // 重新初始化I2C总线
}

void GetIMU(uint8_t address, float& angle_x, float& angle_y, float& angle_z) {
  uint8_t* msg = createGetIMUMsg(address);  // 设置速度PID参数


  Wire.beginTransmission(address);      // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                  // 发送3个字节的数据
  byte error = Wire.endTransmission();  // 关闭通信


  if (error) {
    // 处理错误
    if (error == 2) {
      Serial.println("Device not found or wrong address.");
      Wire.end();
      // delay(100);  // 给一些延时，确保所有操作都已完成
      resetI2Cbus(A4, A5);
      // delay(100);  // 给一些延时，确保所有操作都已完成
      // Wire.begin();
    }
    // ... 其他错误处理
  }

  // Serial.println("Get Speed PID Setting");
  int angle_x_int, angle_y_int, angle_z_int;
  GetIMUPara(address, angle_x_int, angle_y_int, angle_z_int);

  angle_x = float(angle_x_int) / MULTIPLIER;
  angle_y = float(angle_y_int) / MULTIPLIER;
  angle_z = float(angle_z_int) / MULTIPLIER;
}

void InitIMUAngle(uint8_t address) {
  uint8_t* msg = createInitIMUAngleMsg(address);  // 设置速度PID参数

  Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);              // 发送3个字节的数据
  Wire.endTransmission();           // 关闭通信
}

void InitStopPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d) {
  int setting_times = 3;  // 最多设定次数
  for (int i = 0; i < setting_times; i++) {
    uint8_t* msg = createStopPIDMsg(address, motorID, k_p * 100, k_i * 100, k_d * 100);  // 设置速度PID参数
    Wire.beginTransmission(address);                                                     // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);                                                                 // 发送3个字节的数据
    Wire.endTransmission();                                                              // 关闭通信
    delay(10);                                                                           // 短暂延迟

    msg = readI2CMsgStopPID(address, motorID);
    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信
    delay(10);                        // 短暂延迟

    int p_get, i_get, d_get;
    GetPIDPara(address, p_get, i_get, d_get);

    if (int(p_get) == int(k_p * 100) && int(i_get) == int(k_i * 100) && int(d_get) == int(k_d * 100)) {
      Serial.println("Stop PID Setting Complete.");
      break;
    }
    if (i == setting_times - 1) {
      while (1) {
        //        Serial.println("Posi PID Setting Error.");
        delay(1000);
      }
    }
  }
}

void PIDSpeedCtl(uint8_t address, uint8_t motorID, int speed_input, int limit) {
  if (speed_input >= 0) {
    if (speed_input >= limit) {
      speed_input = limit;
    }
  } else {
    if (speed_input <= -limit) {
      speed_input = -limit;
    }
  }
  uint8_t* msg = createI2CMsgEncoderSpeed(address, motorID, speed_input);

  Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);              // 发送3个字节的数据
  Wire.endTransmission();           // 关闭通信

  // delay(2);
}

void PWMSpeedCtl(uint8_t address, uint8_t motorID, int speed_input) {
  uint8_t* msg = createI2CMsgPwmSpeed(address, motorID, speed_input);

  Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);              // 发送3个字节的数据
  Wire.endTransmission();           // 关闭通信

  // delay(2);
}

void SoftPWMSpeedCtl(uint8_t address, uint8_t motorID, int speed1, int speed2, int acc) {
  uint8_t* msg = createI2CSpeedControlMsg(address, motorID, speed1, speed2, acc);

  Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);              // 发送3个字节的数据
  Wire.endTransmission();           // 关闭通信

  // delay(1);
}

void DistanceCtl(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance) {
  for (int retry = 0; retry < MAX_RETRY_TIMES; ++retry) {
    uint8_t* msg = createDistanceMsg(address, motorID, initialSpeed, maxSpeed, acceleration, deceleration, totalDistance);

    uint8_t msg_check = msg[11];

    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信

    // 发送完成后需要再校验下
    msg = readI2CMsgCmd(address);
    Wire.beginTransmission(address);
    Wire.write(msg, 12);
    Wire.endTransmission();

    byte rece_cmd = 99;
    GetCMDPara(address, rece_cmd);

    if (rece_cmd == msg_check) {
      Serial.println("Success!");
      break;
    } else {
      Serial.println(rece_cmd);
      Serial.println(msg_check);
    }
    // delay(5);
  }

  // 阻塞直到完成
  volatile int motor_state = 0;

  while (motor_state < 1) {
    motor_state = GetPosiState(address, motorID);
  }
}

void StopCtl(uint8_t address, uint8_t motorID) {
  for (int retry = 0; retry < MAX_RETRY_TIMES; ++retry) {
    uint8_t* msg = sendI2CMsgStop(address, motorID);

    uint8_t msg_check = msg[11];

    Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);              // 发送3个字节的数据
    Wire.endTransmission();           // 关闭通信

    // 发送之后在获取

    // delay(5);
    // 发送完成后需要再校验下
    msg = readI2CMsgCmd(address);
    Wire.beginTransmission(address);
    Wire.write(msg, 12);
    Wire.endTransmission();

    byte rece_cmd = 99;
    GetCMDPara(address, rece_cmd);

    if (rece_cmd == msg_check) {
      Serial.println("Stop Success!");
      break;
    } else {
      Serial.println(rece_cmd);
      Serial.println(msg_check);
    }
  }
}

// 获取速度
int16_t ReadEncoderSpeed(uint8_t address, uint8_t motorID) {
  uint8_t* msg = readI2CMsgEncoderSpeed(address, motorID);  // 设置速度PID参数
  Wire.beginTransmission(address);                          // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                                      // 发送3个字节的数据
  Wire.endTransmission();                                   // 关闭通信
  // delay(2);                                                 // 短暂延迟

  return GetSpeed(address);
}

// 获取速度
int32_t ReadEncoderDistance(uint8_t address, uint8_t motorID) {
  uint8_t* msg = readI2CMsgEncoderDistance(address, motorID);  // 设置速度PID参数
  Wire.beginTransmission(address);                             // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                                         // 发送3个字节的数据
  Wire.endTransmission();                                      // 关闭通信
  delay(1);                                                    // 短暂延迟

  return GetDistance(address);
}

// 轮子的速度控制
void zuoQian(int ctrl_speed, int target_speed, int acc) {
  // 前左边轮子
  SoftPWMSpeedCtl(STM32_ADDRESS_1, MOTOR_ID_0, target_speed, ctrl_speed, acc);
}

void zuoHou(int ctrl_speed, int target_speed, int acc) {
  // 后左边轮子
  SoftPWMSpeedCtl(STM32_ADDRESS_1, MOTOR_ID_1, target_speed, ctrl_speed, acc);
}

void youHou(int ctrl_speed, int target_speed, int acc) {
  // 后右边轮子
  SoftPWMSpeedCtl(STM32_ADDRESS_0, MOTOR_ID_0, target_speed, ctrl_speed, acc);
}

void youQian(int ctrl_speed, int target_speed, int acc) {
  // 前右边轮子
  SoftPWMSpeedCtl(STM32_ADDRESS_0, MOTOR_ID_1, target_speed, ctrl_speed, acc);
}

// 读取ADC的数值
int16_t readChannel(String channelName) {
  // readChannel() 函数的实现...
  if (channelName.startsWith("EADC")) {
    int channel = channelName.substring(4).toInt();
    channel = 3 - channel;
    if (channel >= 0 && channel <= 3) {
      return (int16_t)(0);  // convert voltage to 0-1024 scale
    }
  } else if (channelName.startsWith("ADC")) {
    int pin = channelName.substring(3).toInt();
    if (pin >= 0 && pin <= 7) {
      return analogRead(pin);
    }
  }

  // Serial.println("Invalid channel name!");
  return 0;
}

uint8_t* sendI2CMsgPIDSetting(uint8_t address, uint8_t pid_setting) {
  // Check if speed is within limits
  // Check if valid motor ID is provided

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = pid_setting;

  // Fill in the unnecessary parameters
  for (int i = 2; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

void PIDSettingOK(uint8_t address, uint8_t pid_setting) {
  uint8_t* msg = sendI2CMsgPIDSetting(address, pid_setting);

  uint8_t msg_check = msg[11];

  Wire.beginTransmission(address);  // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);              // 发送3个字节的数据
  Wire.endTransmission();           // 关闭通信
}

// data ino
// 1、板子复位
uint8_t* createResetMsg(uint8_t address) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;

  // Fill in the unnecessary parameters
  for (int i = 1; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 2、设置使用轮子的直径
uint8_t* createI2CMsgSetDiameter(uint8_t address, uint8_t motorID, int diameter) {
  // Check if speed is within limits
  if (diameter < 0) {
    //    Serial.println("Invalid speed value. Absolute value must be between 0 and 900.");
    return nullptr;
  }

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_WHEEL_DIA;

  // Speed is split into two bytes and its absolute value is used
  cmd[3] = (abs(diameter) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(diameter) & 0xFF;         // Low byte

  // Fill in the unnecessary parameters
  for (int i = 5; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 3、设置速度控制的PID参数
uint8_t* createSpeedPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd) {
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_SPEED_PID_PARA;

  // Kp is split into two bytes and its absolute value is used
  cmd[3] = (abs(Kp) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(Kp) & 0xFF;         // Low byte

  // Ki is split into two bytes and its absolute value is used
  cmd[5] = (abs(Ki) >> 8) & 0xFF;  // High byte
  cmd[6] = abs(Ki) & 0xFF;         // Low byte

  // Kd is split into two bytes and its absolute value is used
  cmd[7] = (abs(Kd) >> 8) & 0xFF;  // High byte
  cmd[8] = abs(Kd) & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  return cmd;
}

// 底盘控制的代码
uint8_t* createChassisGoDistanceMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, 
                                    float angle, float angle_lock, int distance, 
                                    int speed_rate, int init_speed) {
  // 确保speed_rate和init_speed在合法范围内
  if (speed_rate < 0) speed_rate = 0;
  if (speed_rate > 20) speed_rate = 20;
  if (init_speed < 0) init_speed = 0;
  if (init_speed > 100) init_speed = 100;

  // Allocate memory for the message
  static uint8_t cmd[12];

  // Fill in the data
  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  int angle_int = int(angle * 50);
  cmd[3] = ((angle_int) >> 8) & 0xFF;  // High byte
  cmd[4] = (angle_int)&0xFF;           // Low byte

  cmd[5] = ((distance) >> 8) & 0xFF;   // High byte
  cmd[6] = (distance)&0xFF;            // Low byte

  // speed_rate现在是一个整数，直接存储
  cmd[7] = (uint8_t)speed_rate;       

  // init_speed也是一个整数，直接存储
  cmd[8] = (uint8_t)init_speed;       

  int angle_lock_int = int(angle_lock * 50);
  cmd[9] = ((angle_lock_int) >> 8) & 0xFF;  // High byte
  cmd[10] = (angle_lock_int)&0xFF;           // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);

  return cmd;
}

// 底盘控制代码
uint8_t* createVChassisCtrlMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float angle_lock, float w_speed) {
  static uint8_t cmd[12];

  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  // 假设 v_speed 直接以整数形式发送
  cmd[3] = (v_speed >> 8) & 0xFF;  // v_speed 高字节
  cmd[4] = v_speed & 0xFF;         // v_speed 低字节

  // 将 v_angle 和 w_speed 缩放并转换为整数
  int angle_int = int(v_angle * 100);  // 假设缩放因子为 100
  int speed_int = int(w_speed * 100);  // 假设缩放因子为 100
  int angle_int_lock = int(angle_lock * 100);

  cmd[5] = (angle_int >> 8) & 0xFF;  // v_angle 高字节
  cmd[6] = angle_int & 0xFF;         // v_angle 低字节

  cmd[7] = (speed_int >> 8) & 0xFF;  // w_speed 高字节
  cmd[8] = speed_int & 0xFF;         // w_speed 低字节

  cmd[9] = (angle_int_lock >> 8) & 0xFF;  // v_angle 高字节
  cmd[10] = angle_int_lock & 0xFF;         // v_angle 低字节

  // 计算校验和
  cmd[11] = computeXOR(cmd, 11);  // 只计算前9个字节的XOR

  return cmd;
}

void ChassisTurnAngleWoLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle, String turn_state, float speed_rate, GrayScaleSensor sensor) {
  int turn_state_int = 0;
  if(turn_state == "shun"){
    turn_state_int = 1;
  }else if(turn_state == "ni"){
    turn_state_int = 0;
  }
  uint8_t* msg = createChassisTurnAngleWoLockMsg(address, chassis, chassis_cmd, turn_state_int, target_angle, speed_rate);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send ChassisTurnAngleWoLock cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信

  // 要加入到达之后的信号
  // 发送完成
  // 要去获取此时是否到指定的位置
  volatile int chassis_get_state = 0;
  delay(100);

  while ((chassis_get_state) < 1) {
    chassis_get_state = GetChassisPosiState(address, chassis);
    sensor.readWeightedAverage();
  }
}

void ChassisRelease(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle, String turn_state, float speed_rate, GrayScaleSensor sensor) {
  int turn_state_int = 0;
  if(turn_state == "shun"){
    turn_state_int = 1;
  }else if(turn_state == "ni"){
    turn_state_int = 0;
  }
  uint8_t* msg = createChassisTurnAngleWoLockMsg(address, chassis, chassis_cmd, turn_state_int, target_angle, speed_rate);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send ChassisTurnAngleWoLock cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信
}


// 底盘控制代码
uint8_t* createChassisTurnAngleWoLockMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int turn_state, float target_angle, float speed_rate) {
  static uint8_t cmd[12];

  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  // 假设 turn_state 直接以整数形式发送
  cmd[3] = (turn_state >> 8) & 0xFF;  // turn_state 高字节
  cmd[4] = turn_state & 0xFF;         // turn_state 低字节

  // 将 v_angle 和 w_speed 缩放并转换为整数
  int angle_int = int(target_angle * 100);  // 假设缩放因子为 100
  int speed_int = int(speed_rate * 100);  // 假设缩放因子为 100

  cmd[5] = (angle_int >> 8) & 0xFF;  // v_angle 高字节
  cmd[6] = angle_int & 0xFF;         // v_angle 低字节

  cmd[7] = (speed_int >> 8) & 0xFF;  // w_speed 高字节
  cmd[8] = speed_int & 0xFF;         // w_speed 低字节

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // 计算校验和
  cmd[11] = computeXOR(cmd, 11);  // 只计算前9个字节的XOR

  return cmd;
}

void ChassisAngleLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle) {
  Serial.print("target_angle = ");
  Serial.println(target_angle);
  uint8_t* msg = createChassisAngleLockMsg(address, chassis, chassis_cmd, target_angle);  // 设置速度PID参数

  // 发送控制底盘的指令
  Wire.beginTransmission(address);     // 打开与地址为0x01的设备的通信
  Wire.write(msg, 12);                 // 发送3个字节的数据
  Serial.println("Send ChassisAngleLock cmd");  // 打印空格，方便查看每个数据
  Wire.endTransmission();              // 关闭通信

  // 要加入到达之后的信号
}

// 底盘控制代码
uint8_t* createChassisAngleLockMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle) {
  static uint8_t cmd[12];

  cmd[0] = address;
  cmd[1] = chassis;
  cmd[2] = chassis_cmd;

  // 将 v_angle 和 w_speed 缩放并转换为整数
  int angle_int = int(target_angle * 100);  // 假设缩放因子为 100
  Serial.print("angle_int = ");
  Serial.println(angle_int);
  cmd[3] = (angle_int >> 8) & 0xFF;  // v_angle 高字节
  cmd[4] = angle_int & 0xFF;         // v_angle 低字节

  cmd[5] = 0; 
  cmd[6] = 0; 

  cmd[7] = 0; 
  cmd[8] = 0; 

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // 计算校验和
  cmd[11] = computeXOR(cmd, 11);  // 只计算前9个字节的XOR

  return cmd;
}

// 创建获取IMU的message
uint8_t* createGetIMUMsg(uint8_t address) {

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = SEND_ANGLE_DATA;
  cmd[2] = SEND_ANGLE_DATA;

  // Kp is split into two bytes and its absolute value is used
  cmd[3] = 0;  // High byte
  cmd[4] = 0;  // Low byte

  // Ki is split into two bytes and its absolute value is used
  cmd[5] = 0;  // High byte
  cmd[6] = 0;  // Low byte

  // Kd is split into two bytes and its absolute value is used
  cmd[7] = 0;  // High byte
  cmd[8] = 0;  // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  return cmd;
}

uint8_t* createInitIMUAngleMsg(uint8_t address) {

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = INIT_ANGLE;
  cmd[2] = INIT_ANGLE;

  // Kp is split into two bytes and its absolute value is used
  cmd[3] = 0;  // High byte
  cmd[4] = 0;  // Low byte

  // Ki is split into two bytes and its absolute value is used
  cmd[5] = 0;  // High byte
  cmd[6] = 0;  // Low byte

  // Kd is split into two bytes and its absolute value is used
  cmd[7] = 0;  // High byte
  cmd[8] = 0;  // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  return cmd;
}

// 4、读取速度控制的PID参数
uint8_t* readI2CMsgSpeedPID(uint8_t address, uint8_t motorID) {
  // Check if speed is within limits

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_SPEED_PID_PARA;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 判断指令是否已经到达
uint8_t* readI2CMsgCmd(uint8_t address) {
  // Check if speed is within limits

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = 0;
  cmd[2] = READ_SEND_CMD;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 处理解析陀螺仪的数据
bool parseIMUData(byte* RxData, int16_t* angle_x, int16_t* angle_y, int16_t* angle_z) {
  byte checkSum = computeXOR(RxData, 7);

  if (checkSum != RxData[7]) {
    return false;
  }

  *angle_x = (int16_t)((RxData[1] << 8) | RxData[2]);
  *angle_y = (int16_t)((RxData[3] << 8) | RxData[4]);
  *angle_z = (int16_t)((RxData[5] << 8) | RxData[6]);

  return true;
}

// 4、接收处理函数解析函数
bool parseData(byte* RxData, int* Kp, int* Ki, int* Kd) {
  // 计算XOR校验
  byte checkSum = computeXOR(RxData, 7);

  // 检验校验位
  if (checkSum != RxData[7]) {
    return false;
  }

  // 提取Kp
  *Kp = (RxData[1] << 8) | RxData[2];

  // 提取Ki
  *Ki = (RxData[3] << 8) | RxData[4];

  // 提取Kd
  *Kd = (RxData[5] << 8) | RxData[6];

  return true;
}

// 接收IMU处理函数
// 4、接收处理函数
void GetIMUPara(uint8_t address, int& angle_x, int& angle_y, int& angle_z) {
  // 请求来自从设备的数据，假设我们请求3个字节
  byte RxData[8];  // 用来存储接收到的数据
  int index = 0;   // 用来记录当前存储到了哪个位置

  Wire.requestFrom(address, 8);
  while (Wire.available() && index < 8)  // 从设备发送的字节仍在等待时
  {
    RxData[index] = Wire.read();  // 通过I2C总线接收一个字符
                                  //    Serial.print("Received: ");
                                  //    Serial.println(RxData[index], HEX); // 打印接收到的数据
    index++;
  }

  // 如果接收到了足够的数据，就进行解析
  if (index == 8) {
    int16_t angle_x_int16, angle_y_int16, angle_z_int16;
    if (parseIMUData(RxData, &angle_x_int16, &angle_y_int16, &angle_z_int16)) {
      // 校验正确，可以使用解析出的参数
      angle_x = (int)angle_x_int16;
      angle_y = (int)angle_y_int16;
      angle_z = (int)angle_z_int16;
      // ... 其他代码 ...
    } else {
      // 校验错误，数据可能有误
      // Serial.println("Checksum error!");
    }
  }
}

// 4、接收处理函数
void GetPIDPara(uint8_t address, int& Kp, int& Ki, int& Kd) {
  // 请求来自从设备的数据，假设我们请求3个字节
  byte RxData[8];  // 用来存储接收到的数据
  int index = 0;   // 用来记录当前存储到了哪个位置

  Wire.requestFrom(address, 8);
  while (Wire.available() && index < 8)  // 从设备发送的字节仍在等待时
  {
    RxData[index] = Wire.read();  // 通过I2C总线接收一个字符
                                  //    Serial.print("Received: ");
                                  //    Serial.println(RxData[index], HEX); // 打印接收到的数据
    index++;
  }

  // 如果接收到了足够的数据，就进行解析
  if (index == 8) {
    if (parseData(RxData, &Kp, &Ki, &Kd)) {
      // 校验正确，可以使用解析出的参数
      //      Serial.print("Parsed Kp: ");
      //      Serial.println(Kp/100);
      //      Serial.print("Parsed Ki: ");
      //      Serial.println(Ki/100);
      //      Serial.print("Parsed Kd: ");
      //      Serial.println(Kd/100);
    } else {
      // 校验错误，数据可能有误
      //      Serial.println("Checksum error!");
    }
  }
}

void GetCMDPara(uint8_t address, byte& cmd) {
  // 请求来自从设备的数据，假设我们请求3个字节
  byte RxData[8];  // 用来存储接收到的数据
  int index = 0;   // 用来记录当前存储到了哪个位置

  Wire.requestFrom(address, 8);

  unsigned long startMillis = millis();  // 获取当前时间
  while (Wire.available() && index < 8)  // 从设备发送的字节仍在等待时
  {
    if (millis() - startMillis > TIMEOUT_MS) {
      // 如果已经超时，跳出循环
      Serial.println("Timeout!");
      cmd = 99;
      return;
    }

    RxData[index] = Wire.read();  // 通过I2C总线接收一个字符
    index++;
  }

  // 如果接收到了足够的数据，就进行解析
  if (index == 8) {
    byte checkSum = computeXOR(RxData, 7);

    // 检验校验位
    if (checkSum != RxData[7]) {
      cmd = 99;
      return;
    }

    cmd = RxData[1];
  }
}

// 5、设置位置控制的PID参数
uint8_t* createPosiPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd) {
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_POSI_PID_PARA;

  // Kp is split into two bytes and its absolute value is used
  cmd[3] = (abs(Kp) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(Kp) & 0xFF;         // Low byte

  // Ki is split into two bytes and its absolute value is used
  cmd[5] = (abs(Ki) >> 8) & 0xFF;  // High byte
  cmd[6] = abs(Ki) & 0xFF;         // Low byte

  // Kd is split into two bytes and its absolute value is used
  cmd[7] = (abs(Kd) >> 8) & 0xFF;  // High byte
  cmd[8] = abs(Kd) & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 6、读取位置控制的PID参数
uint8_t* readI2CMsgPosiPID(uint8_t address, uint8_t motorID) {
  // Check if speed is within limits

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_POSI_PID_PARA;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 7、读取电机的实时编码器速度
uint8_t* readI2CMsgEncoderSpeed(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_REALTIME_ENCODER_SPEED;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 8、读取行驶的实时速度
uint8_t* readI2CMsgRealtimeSpeed(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_RUNNING_ENCODER_SPEED;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 9、读取电机的累计编码器数值
uint8_t* readI2CMsgEncoderDistance(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_ENCODER_CNT;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 10、读取电机的累计绝对编码器数值
uint8_t* readI2CMsgAbsEncoderDistance(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_ABS_ENCODER_CNT;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 11、读取行驶的累计距离
uint8_t* readI2CMsgDistance(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_RUN_DISTANCE_CNT;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 12、读取行驶的绝对累计距离
uint8_t* readI2CMsgAbsDistance(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_ABS_RUN_DISTANCE_CNT;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 13、控制电机的PWM输出大小
uint8_t* createI2CMsgPwmSpeed(uint8_t address, uint8_t motorID, int speed) {
  // Check if speed is within limits
  if (abs(speed) > 1023) {
    //    Serial.println("Invalid speed value. Absolute value must be between 0 and 900.");
    speed = (speed / abs(speed)) * 1023;
  }

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_PWM_SPEED;

  // Speed is split into two bytes and its absolute value is used
  cmd[3] = (abs(speed) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(speed) & 0xFF;         // Low byte

  // Fill in the direction based on the sign of the speed
  cmd[5] = speed >= 0 ? 0x00 : 0x01;

  // Fill in the unnecessary parameters
  for (int i = 6; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 14、控制电机的编码器PID速度
uint8_t* createI2CMsgEncoderSpeed(uint8_t address, uint8_t motorID, int speed) {
  // Check if speed is within limits
  if (abs(speed) > 120) {
    //    Serial.println("Invalid speed value. Absolute value must be between 0 and 120.");
    return nullptr;
  }

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_ENCODER_SPEED;

  // Speed is split into two bytes and its absolute value is used
  cmd[3] = (abs(speed) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(speed) & 0xFF;         // Low byte

  // Fill in the direction based on the sign of the speed
  cmd[5] = speed >= 0 ? 0x00 : 0x01;

  // Fill in the unnecessary parameters
  for (int i = 6; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 15、控制电机转动一定的行驶距离
uint8_t* createDistanceMsg(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance) {
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }
  //  Serial.print("Motor");
  //  Serial.println(motorID);
  // Allocate memory for the message
  static uint8_t cmd[12];

  // Fill in the data
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_GO_DISTANCE;
  cmd[3] = initialSpeed;
  cmd[4] = maxSpeed;
  cmd[5] = acceleration;
  cmd[6] = deceleration;

  // Total distance is split into two bytes
  cmd[7] = (totalDistance >> 8) & 0xFF;  // High byte
  cmd[8] = totalDistance & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 16、控制电机转动一定的编码器数值
uint8_t* createEncoderDistanceMsg(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance) {
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // Fill in the data
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_GO_ENCODER;
  cmd[3] = initialSpeed;
  cmd[4] = maxSpeed;
  cmd[5] = acceleration;
  cmd[6] = deceleration;

  // Total distance is split into two bytes
  cmd[7] = (totalDistance >> 8) & 0xFF;  // High byte
  cmd[8] = totalDistance & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 17、读取判断是否到达目标距离位置
uint8_t* readI2CMsgHaveGotTargetPosi(uint8_t address, uint8_t motorID) {
  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_HAVE_GOT_POSI;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 18、速度控制两段柔性变化
uint8_t* createI2CSpeedControlMsg(uint8_t address, uint8_t motorID, int speed1, int speed2, int acc) {
  static uint8_t data[10];

  // 填充数据数组
  data[0] = address;
  data[1] = motorID;
  data[2] = SET_TWO_DISTANCE;

  // Speed1
  data[3] = (speed1 >> 8) & 0xFF;  // High byte
  data[4] = speed1 & 0xFF;         // Low byte

  // Speed2
  data[5] = (speed2 >> 8) & 0xFF;  // High byte
  data[6] = speed2 & 0xFF;         // Low byte

  // Remaining bytes are 0
  data[7] = (acc >> 8) & 0xFF;  // High byte
  data[8] = acc & 0xFF;         // Low byte

  data[9] = 0;  // High byte
  data[10] = 0;         // Low byte

  // 计算异或校验和
  data[11] = 0;
  for (int i = 0; i < 11; i++) {
    data[11] ^= data[i];
  }

  return data;
}

// 19、设定一圈编码器数值
uint8_t* createI2CMsgSetCircleCnt(uint8_t address, uint8_t motorID, int circle_cnt) {
  // Check if speed is within limits
  if (circle_cnt < 0) {
    //    Serial.println("Invalid speed value. Absolute value must be between 0 and 900.");
    return nullptr;
  }

  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_CIRCLE_ENCODER;

  // Speed is split into two bytes and its absolute value is used
  cmd[3] = (abs(circle_cnt) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(circle_cnt) & 0xFF;         // Low byte

  // Fill in the unnecessary parameters
  for (int i = 5; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 20、设定刹车的位置PID参数
uint8_t* createStopPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd) {
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_STOP_PID;

  // Kp is split into two bytes and its absolute value is used
  cmd[3] = (abs(Kp) >> 8) & 0xFF;  // High byte
  cmd[4] = abs(Kp) & 0xFF;         // Low byte

  // Ki is split into two bytes and its absolute value is used
  cmd[5] = (abs(Ki) >> 8) & 0xFF;  // High byte
  cmd[6] = abs(Ki) & 0xFF;         // Low byte

  // Kd is split into two bytes and its absolute value is used
  cmd[7] = (abs(Kd) >> 8) & 0xFF;  // High byte
  cmd[8] = abs(Kd) & 0xFF;         // Low byte

  cmd[9] = 0;  // High byte
  cmd[10] = 0;         // Low byte

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 21、读取刹车的位置PID参数
uint8_t* readI2CMsgStopPID(uint8_t address, uint8_t motorID) {
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = READ_STOP_PID_PARA;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 22、设定刹车
uint8_t* sendI2CMsgStop(uint8_t address, uint8_t motorID) {
  // Check if speed is within limits
  // Check if valid motor ID is provided
  if (motorID != 0x01 && motorID != 0x02 && motorID != 0x03 && motorID != 0x04) {
    //    Serial.println("Invalid motor ID. Must be 0x01 or 0x02.");
    return nullptr;
  }

  // Allocate memory for the message
  static uint8_t cmd[12];

  // First, fill in the data as specified
  cmd[0] = address;
  cmd[1] = motorID;
  cmd[2] = SET_STOP_NOW;

  // Fill in the unnecessary parameters
  for (int i = 3; i < 11; i++) {
    cmd[i] = 0;
  }

  // Compute and set the XOR checksum
  cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

  // Return the created message
  return cmd;
}

// 异或计算
byte computeXOR(uint8_t* data, int length) {
  byte checksum = 0;
  for (int i = 0; i < length; i++) {
    checksum ^= data[i];
  }
  return checksum;
}

int32_t GetDistance(uint8_t address) {
  Wire.requestFrom(address, DISTANCE_MESSAGE_LENGTH);

  uint8_t msg[DISTANCE_MESSAGE_LENGTH];
  for (int i = 0; i < DISTANCE_MESSAGE_LENGTH; i++) {
    msg[i] = Wire.read();
  }

  // Verify the XOR checksum
  uint8_t receivedChecksum = msg[DISTANCE_MESSAGE_LENGTH - 1];
  uint8_t computedChecksum = computeXOR(msg, DISTANCE_MESSAGE_LENGTH - 1);  // Exclude the checksum byte from the computation
  if (receivedChecksum != computedChecksum) {
    //    Serial.println("Invalid XOR checksum.");
    // 整个地方有问题
    return -1;
  }

  // Extract and combine the distance
  int32_t distance = ((int32_t)msg[1] << 24) | ((int32_t)msg[2] << 16) | ((int32_t)msg[3] << 8) | (int32_t)msg[4];

  //  Serial.print("Received distance: ");
  //  // serial.println(distance);
  return distance;
}


int16_t GetSpeed(uint8_t address) {
  Wire.requestFrom(address, DISTANCE_MESSAGE_LENGTH);

  uint8_t msg[SPEED_MESSAGE_LENGTH];
  for (int i = 0; i < SPEED_MESSAGE_LENGTH; i++) {
    msg[i] = Wire.read();
  }

  // Verify the XOR checksum
  uint8_t receivedChecksum = msg[SPEED_MESSAGE_LENGTH - 1];
  uint8_t computedChecksum = computeXOR(msg, SPEED_MESSAGE_LENGTH - 1);  // Exclude the checksum byte from the computation
  if (receivedChecksum != computedChecksum) {
    //    // serial.println("Invalid XOR checksum.");
    // 这里有修改过
    return -1;
  }

  // Extract and combine the distance
  int16_t distance = ((int32_t)msg[1] << 8) | (int32_t)msg[2];
  //  // serial.print("Received speed: ");
  //  // serial.println(distance);
  return distance;
}

float getZAngle() {
  float real_time_angle_x, real_time_angle_y, real_time_angle_z;
  GetIMU(STM32_ADDRESS_0, real_time_angle_x, real_time_angle_y, real_time_angle_z);
  return real_time_angle_z;
}

Position decodePosition(const uint8_t* response) {
    Position pos;

    // 解析posi_x
    pos.x = ((int32_t)response[1] << 16) | ((int32_t)response[2] << 8) | response[3];
    // 检查符号位并扩展到32位
    if (pos.x & 0x80000) pos.x |= 0xFFF00000;

    // 解析posi_y
    pos.y = ((int32_t)response[4] << 16) | ((int32_t)response[5] << 8) | response[6];
    // 检查符号位并扩展到32位
    if (pos.y & 0x80000) pos.y |= 0xFFF00000;

    pos.x = -int(float(pos.x) / 4.414);
    pos.y = -int(float(pos.y) / 4.414);

    return pos;
}

Position ReadChassisDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd) {
    uint8_t* msg = readI2CMsgChassisDistance(address, chassis, chassis_cmd);  // 设置速度PID参数
    Wire.beginTransmission(address);                             // 打开与地址为0x01的设备的通信
    Wire.write(msg, 12);                                         // 发送3个字节的数据
    Wire.endTransmission();                                      // 关闭通信
    delay(1);                                                    // 短暂延迟

    // 假设response是从I2C读取回来的数据
    uint8_t response[8];
    Wire.requestFrom((int)address, 8);  // 请求8个字节的数据
    for (int i = 0; i < 8; i++) {
        response[i] = Wire.read();      // 读取数据
    }

    return decodePosition(response);    // 解析位置信息
}

uint8_t* readI2CMsgChassisDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd) {
    // Allocate memory for the message
    static uint8_t cmd[12];

    // Fill in the data as specified
    cmd[0] = address;
    cmd[1] = chassis;
    cmd[2] = chassis_cmd;

    // Fill in the unnecessary parameters
    for (int i = 3; i < 11; i++) {
        cmd[i] = 0;
    }

    // Compute and set the XOR checksum
    cmd[11] = computeXOR(cmd, 11);  // Only compute XOR for the first 9 bytes

    // Return the created message
    return cmd;
}

// Parameter
Parameter::Parameter(const String& displayName, int eepromAddr, void (*function)(), bool isFloat)
  : name(displayName), addr(eepromAddr), executeFunction(function), isFloat(isFloat) {
  int intValue;
  EEPROM.get(addr, intValue);
  value = isFloat ? intValue / 10.0 : intValue;
}

void Parameter::increase() {
  value += isFloat ? 0.1 : 1;
  EEPROM.put(addr, isFloat ? (int)(value * 10) : value);
}

void Parameter::decrease() {
  value -= isFloat ? 0.1 : 1;
  EEPROM.put(addr, isFloat ? (int)(value * 10) : value);
}

float Parameter::getValue() {
  return value;
}

String Parameter::getName() {
  return name;
}

void Parameter::execute() {
  executeFunction();
}

bool Parameter::isFloatValue() {
  return isFloat;
}

// LineSensorArrayNew
LineSensorArrayNew::LineSensorArrayNew(int num, int* pins, float* weights, int* addresses, String color_state_in)
  : numSensors(num), sensorNames(pins), lastLinePosition(0.0), color_state(color_state_in) {
  sensorThresholds = new int[numSensors];
  sensorWeights = new float[numSensors];
  eepromAddresses = new int[numSensors];

  // 设置每个传感器的引脚为输入模式
  for (int i = 0; i < numSensors; i++) {
    pinMode(sensorNames[i], INPUT);
    sensorWeights[i] = weights[i];
    eepromAddresses[i] = addresses[i];
  }
}

void LineSensorArrayNew::initMinMaxValue(float min_value_input, float max_value_input) {
  min_value = min_value_input;
  max_value = max_value_input;
}

int16_t LineSensorArrayNew::readChannel(int pin) {
  // readChannel() 函数的实现...
  if (pin >= 0 && pin <= 53) {  // Arduino Mega最多53个数字引脚
    if (digitalRead(pin) == HIGH) {
      if(color_state == "white_bg_black_line"){
        return 0; 
      }else{
        return 1; 
      }
      
    } else {
      if(color_state == "white_bg_black_line"){
        return 1; 
      }else{
        return 0; 
      }
    }
  }
  return 0;
}

void LineSensorArrayNew::calculateThresholds(int numReadings) {
  // calculateThresholds() 函数的实现...
  int* minValues = new int[numSensors];
  int* maxValues = new int[numSensors];

  for (int i = 0; i < numSensors; i++) {
    minValues[i] = INT16_MAX;
    maxValues[i] = INT16_MIN;
  }

  for (int j = 0; j < numReadings; j++) {
    delay(20);
    for (int i = 0; i < numSensors; i++) {
      int sensorPin = sensorNames[i];
      int16_t sensorValue = readChannel(sensorPin);
      if (sensorValue < minValues[i]) {
        minValues[i] = sensorValue;
      }
      if (sensorValue > maxValues[i]) {
        maxValues[i] = sensorValue;
      }
    }
  }

  for (int i = 0; i < numSensors; i++) {
    sensorThresholds[i] = (minValues[i] + maxValues[i]) / 2;
  }

  delete[] minValues;
  delete[] maxValues;

  for (int i = 0; i < numSensors; i++) {
    int address = eepromAddresses[i];
    EEPROM.put(address, sensorThresholds[i]);
  }
}

void LineSensorArrayNew::loadThresholdsFromEEPROM(int edit_value[]) {
  // loadThresholdsFromEEPROM() 函数的实现...
  for (int i = 0; i < numSensors; i++) {
    int address = eepromAddresses[i];
    EEPROM.get(address, sensorThresholds[i]);
    sensorThresholds[i] = 500;  // 直接采用一个定制
  }
}

bool LineSensorArrayNew::getSingleSensor(int i) {
  // getSingleSensor() 函数的实现...
  int sensorPin = sensorNames[i];
  int sensorReadings = readChannel(sensorPin);
  if (sensorReadings) {
    return true;
  }
  return false;
}

float LineSensorArrayNew::getLinePosition() {
  // getLinePosition() 函数的实现...
  int* sensorReadings = new int[numSensors];
  for (int i = 0; i < numSensors; i++) {
    int sensorPin = sensorNames[i];
    sensorReadings[i] = readChannel(sensorPin);
  }

  int numDetected = 0;
  float weightedSum = 0.0;

  for (int i = 0; i < numSensors; i++) {
    if (sensorReadings[i]) {
      numDetected++;
      weightedSum += sensorWeights[i];
    }
  }

  delete[] sensorReadings;

  if (numDetected > 0) {
    lastLinePosition = weightedSum / numDetected;
    return lastLinePosition;
  } else {
    if (lastLinePosition < 0) {
      lastLinePosition = min_value;
    } else if (lastLinePosition > 0) {
      lastLinePosition = max_value;
    }
    return lastLinePosition;
  }
}

int* LineSensorArrayNew::getThresholds() {
  return sensorThresholds;
}

void LineSensorArrayNew::updateSensorWeights(float* newSensorWeights) {
  for (int i = 0; i < numSensors; i++) {
    sensorWeights[i] = newSensorWeights[i];
  }
}

PID::PID(float kp, float ki, float kd, float minInte, float maxInte)
  : Kp(kp), Ki(ki), Kd(kd), erroInte(0), errorLast(0), minErroInte(minInte), maxErroInte(maxInte) {}

int PID::calcPid(float input) {
  float error, errorDiff, output;
  error = input;
  errorDiff = error - errorLast;
  erroInte = constrain(erroInte + error, minErroInte, maxErroInte);
  output = Kp * error + Ki * erroInte + Kd * errorDiff;
  errorLast = error;
  return int(output);  // Ensure the output is an integer.
}
