#ifndef DEDEDCCARBASE_V1_0_H
#define DEDEDCCARBASE_V1_0_H

#include <Adafruit_NeoPixel.h>
#include <SoftwareSerial.h>  // 引入SoftwareSerial库
#include <stdint.h>
#include <U8x8lib.h>
#include <Adafruit_INA219.h>
#include <EEPROM.h>
#include <Wire.h>
#include <Servo.h>
#include <Wire.h>
#include <SoftwareSerial.h>
#include <math.h>
#include <Wire.h>




// 感为的传感器

/* 默认地址 */
#define GW_GRAY_ADDR_DEF 0x4C
#define GW_GRAY_PING 0xAA
#define GW_GRAY_PING_OK 0x66
#define GW_GRAY_PING_RSP GW_GRAY_PING_OK

/* 开启开关数据模式 */
#define GW_GRAY_DIGITAL_MODE 0xDD

/* 开启连续读取模拟数据模式 */
#define GW_GRAY_ANALOG_BASE_ 0xB0
#define GW_GRAY_ANALOG_MODE (GW_GRAY_ANALOG_BASE_ + 0)

/* 传感器归一化寄存器(v3.6及之后的固件) */
#define GW_GRAY_ANALOG_NORMALIZE 0xCF

/* 循环读取单个探头模拟数据 n从1开始到8 */
#define GW_GRAY_ANALOG(n) (GW_GRAY_ANALOG_BASE_ + (n))

/* 黑色滞回比较参数操作 */
#define GW_GRAY_CALIBRATION_BLACK 0xD0
/* 白色滞回比较参数操作 */
#define GW_GRAY_CALIBRATION_WHITE 0xD1

// 设置所需探头的模拟信号(CE: channel enable)
#define GW_GRAY_ANALOG_CHANNEL_ENABLE 0xCE
#define GW_GRAY_ANALOG_CH_EN_1 (0x1 << 0)
#define GW_GRAY_ANALOG_CH_EN_2 (0x1 << 1)
#define GW_GRAY_ANALOG_CH_EN_3 (0x1 << 2)
#define GW_GRAY_ANALOG_CH_EN_4 (0x1 << 3)
#define GW_GRAY_ANALOG_CH_EN_5 (0x1 << 4)
#define GW_GRAY_ANALOG_CH_EN_6 (0x1 << 5)
#define GW_GRAY_ANALOG_CH_EN_7 (0x1 << 6)
#define GW_GRAY_ANALOG_CH_EN_8 (0x1 << 7)
#define GW_GRAY_ANALOG_CH_EN_ALL (0xFF)

/* 读取错误信息 */
#define GW_GRAY_ERROR 0xDE

/* 设备软件重启 */
#define GW_GRAY_REBOOT 0xC0

/* 读取固件版本号 */
#define GW_GRAY_FIRMWARE 0xC1


/**
 * @brief 从I2C得到的8位的数字信号的数据 读取第n位的数据
 * @param sensor_value_8 数字IO的数据
 * @param n 第1位从1开始, n=1 是传感器的第一个探头数据, n=8是最后一个
 * @return
 */
#define GET_NTH_BIT(sensor_value, nth_bit) (((sensor_value) >> ((nth_bit)-1)) & 0x01)

/**
 * @brief 从一个变量分离出所有的bit
 */
#define SEP_ALL_BIT8(sensor_value, val1, val2, val3, val4, val5, val6, val7, val8) \
  do { \
    val1 = GET_NTH_BIT(sensor_value, 1); \
    val2 = GET_NTH_BIT(sensor_value, 2); \
    val3 = GET_NTH_BIT(sensor_value, 3); \
    val4 = GET_NTH_BIT(sensor_value, 4); \
    val5 = GET_NTH_BIT(sensor_value, 5); \
    val6 = GET_NTH_BIT(sensor_value, 6); \
    val7 = GET_NTH_BIT(sensor_value, 7); \
    val8 = GET_NTH_BIT(sensor_value, 8); \
  } while (0)

/* 设置设备I2C地址 */
#define GW_GRAY_CHANGE_ADDR 0xAD

/* 广播重置地址所需要发的数据 */
#define GW_GRAY_BROADCAST_RESET "\xB8\xD0\xCE\xAA\xBF\xC6\xBC\xBC"

#define L 95
#define A 37 * PI / 180.0

#define PIN 13      // 你连接LED灯带的Arduino引脚号
#define NUM_LEDS 1  // 你的LED灯带上的LED数量

// 要补充上校验的部分
#define MAX_RETRY_TIMES 3

#define STM32_ADDRESS_0 0x72
#define STM32_ADDRESS_1 0x72

#define MOTOR_ID_0 0x01
#define MOTOR_ID_1 0x02
#define MOTOR_ID_2 0x03
#define MOTOR_ID_3 0x04

#define PID_STATE 0x19

#define RESET_CMD 0x00                    // 板子复位
#define SET_WHEEL_DIA 0x01                // 设置使用轮子的直径
#define SET_SPEED_PID_PARA 0x02           // 设置速度控制的PID参数
#define READ_SPEED_PID_PARA 0x03          // 读取速度控制的PID参数
#define SET_POSI_PID_PARA 0x04            // 设置位置控制的PID参数
#define READ_POSI_PID_PARA 0x05           // 读取位置控制的PID参数
#define READ_REALTIME_ENCODER_SPEED 0x06  // 读取电机的实时编码器速度
#define READ_RUNNING_ENCODER_SPEED 0x07   // 读取行驶的实时速度
#define READ_ENCODER_CNT 0x08             // 读取电机的累计编码器数值
#define READ_ABS_ENCODER_CNT 0x09         // 读取电机的累计绝对编码器数值
#define READ_RUN_DISTANCE_CNT 0x0A        // 读取行驶的累计距离
#define READ_ABS_RUN_DISTANCE_CNT 0x0B    // 读取行驶的绝对累计距离
#define SET_PWM_SPEED 0x0C                // 控制电机的PWM输出大小
#define SET_ENCODER_SPEED 0x0D            // 控制电机的编码器PID速度
#define SET_GO_DISTANCE 0x0E              // 控制电机转动一定的行驶距离
#define SET_GO_ENCODER 0x0F               // 控制电机转动一定的编码器数值
#define READ_HAVE_GOT_POSI 0x10           // 读取判断是否到达目标距离位置
#define SET_TWO_DISTANCE 0x11             // 速度控制两段柔性变化
#define SET_CIRCLE_ENCODER 0x12           // 设定一圈编码器数值
#define SET_STOP_PID 0x13                 // 设定刹车的位置PID参数
#define READ_STOP_PID_PARA 0x14           // 读取速度控制的PID参数
#define SET_STOP_NOW 0x15                 // 启动刹车
#define READ_SEND_CMD 0x16                // 读取指令
#define SEND_ANGLE_DATA 0x17              // 获取角度
#define INIT_ANGLE 0x18                   // 初始化陀螺仪
#define MULTIPLIER 100

// 底盘的控制指令
#define CHASSIS             0x20          // 对底盘的整体控制
#define CHASSIS_GO_DISTANCE 0x21          // 底盘朝着某个角度移动多少的距离
#define CHASSIS_TURN_ANGLE  0x22          // 底盘的头转动到某个角度
#define CHASSIS_READ_HAVE_GOT_POSI 0x23   // 底盘是否已经达到目标位置
#define CHASSIS_V_CTRL 0x24               // 底盘的速度控制指令
#define CHASSIS_V_CTRL_ANGLE_LOCK 0x25    // 底盘的速度控制指令，角度锁定
#define CHASSIS_TURN_ANGLE_WO_LOCK 0x26   // 底盘的速度控制指令，角度锁定
#define CHASSIS_TURN_ANGLE_WITH_LOCK 0x27 // 底盘的速度控制指令，角度锁定
#define CHASSIS_ANGLE_LOCK 0x28     
#define CHASSIS_STOP 0x29     
#define CHASSIS_STRAIGHT_LINE_ANGLE 0x30 // 直走保持角度PID参数
#define CHASSIS_STRAIGHT_LINE_ANGLE_READ 0x31 // 直走保持角度PID参数
#define CHASSIS_Position_Correction 0x32 // 
#define CHASSIS_Position_Correction_READ 0x33 // 
#define CHASSIS_Turn_Angle_Lock 0x34
#define CHASSIS_Turn_Angle_Lock_READ 0x35
#define CHASSIS_READ_POSI_STATE 0x36
#define CHASSIS_RELEASE 0x37


#define LED_PIN 13
#define NUM_LEDS 1
#define COLOR_ORDER GRB
#define CHIPSET WS2812B

#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 32  // OLED display height, in pixels

#define BUTTON_PIN_1 12  // Increase parameter
#define BUTTON_PIN_2 7   // Decrease parameter
#define BUTTON_PIN_3 8   // Switch parameter or enter function

// EEPROM addresses for parameters
#define PARAMETER_1_ADDR 0
#define PARAMETER_2_ADDR sizeof(int)
#define PARAMETER_3_ADDR sizeof(int) * 2
#define PARAMETER_4_ADDR sizeof(int) * 3
#define PARAMETER_5_ADDR sizeof(int) * 4
#define PARAMETER_6_ADDR sizeof(int) * 5
#define PARAMETER_7_ADDR sizeof(int) * 6
#define PARAMETER_8_ADDR sizeof(int) * 7
#define PARAMETER_9_ADDR sizeof(int) * 8
#define PARAMETER_10_ADDR sizeof(int) * 9
#define PARAMETER_11_ADDR sizeof(int) * 10
#define PARAMETER_12_ADDR sizeof(int) * 11

#define GW_GRAY_ADDR 0x4C  // 使用默认地址
#define GW_GRAY_ADDR_QIAN 0x4D  // 使用默认地址


#define READ_SPEED_PID_PARA 0x03  // 读取速度控制的PID参数

#define TIMEOUT_MS 5  // 设定超时时间，例如500毫秒

#define DISTANCE_MESSAGE_LENGTH 8

#define SPEED_MESSAGE_LENGTH 8

class MyNeoPixel {
private:
  Adafruit_NeoPixel strip;
public:
  MyNeoPixel(uint16_t numLeds, uint8_t pin);
  void begin();
  void colorWipe(uint32_t color);
  void red();
  void green();
  void blue();
  void white();
  void green_blue();
  void turnOff();
  void light();
};

class SimpleServoControl {
private:
  int servo_sdata[10];
  SoftwareSerial* serial;                                                    
  int u_numm;
  int cur_mode;            
  int run_time;            
  double servo_rpara[10];  
  double exp_angle;        
  double cur_angle;
  int servo_recvdata[16];
  int id_number;

  void senddata();

public:
  SimpleServoControl(SoftwareSerial& serialRef);
  void set_angle(int id_num, float angle, int steps);
  float get_state(int id_num, int para_num, int o_m);
};


// 走距离是阻塞的，走距离的控制
struct Motor {
  uint8_t device_id;
  uint8_t motor_id;
  int16_t distance;
};

struct Position {
    int32_t x;
    int32_t y;
};

class GrayScaleSensor {
private:
  uint8_t address;
  float weights[8];
  float lastWeightedAverage = 0.0;
  bool isInitialized = false; // 添加初始化标志

public:
  // 添加默认构造函数
  GrayScaleSensor() : 
    address(GW_GRAY_ADDR), 
    lastWeightedAverage(0.0),
    isInitialized(false) {
    // 初始化权重数组
    for (int i = 0; i < 8; i++) {
      weights[i] = 1.0;
    }
  }
  
  // 保留原有构造函数，但标记为已初始化
  GrayScaleSensor(uint8_t addr) : 
    address(addr), 
    lastWeightedAverage(0.0),
    isInitialized(true) {
    // 初始化权重数组
    for (int i = 0; i < 8; i++) {
      weights[i] = 1.0;
    }
  }
  
  // 添加初始化方法
  void begin(uint8_t addr = GW_GRAY_ADDR) {
    address = addr;
    isInitialized = true;
  }
  
  // 修改现有方法，检查初始化状态
  void setWeights(float new_weights[8]) {
    for (int i = 0; i < 8; i++) {
      weights[i] = new_weights[i];
    }
  }
  
  bool ping() {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return false;
    }
    
    uint8_t ping_rep = 0;
    while (ping_rep != GW_GRAY_PING_OK) {
      Wire.beginTransmission(address);
      Wire.write(GW_GRAY_PING);
      Wire.endTransmission();

      Wire.requestFrom(address, 1);
      ping_rep = Wire.read();
      delay(1);
    }
    return ping_rep == GW_GRAY_PING_OK;
  }
  
  void setDigitalMode() {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return;
    }
    
    Wire.beginTransmission(address);
    Wire.write(GW_GRAY_DIGITAL_MODE);
    Wire.endTransmission();
  }
  
  uint8_t read() {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return 0;
    }
    
    Wire.requestFrom(address, 1);
    return Wire.read();
  }
  
  float readWeightedAverage() {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return 0.0;
    }
    
    uint8_t recv_value = read();
    float sum = 0;
    int count = 0;
    for (int i = 0; i < 8; ++i) {
      if (((recv_value >> i) & 1) == 0) {
        sum += weights[i];
        count++;
      }
    }
    if (count > 0) {
      lastWeightedAverage = sum / count;
      return lastWeightedAverage;
    } else {
      return lastWeightedAverage;
    }
  }
  
  void printSensorStatus() {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return;
    }
    
    uint8_t recv_value = read();
    Serial.print("传感器数字数据:");
    for (int i = 0; i < 8; ++i) {
      Serial.print(" ");
      Serial.print((recv_value >> i) & 1);
    }
    Serial.print("\n");
  }
  
  uint8_t readSensorState(int index) {
    if (!isInitialized) {
      Serial.println("Error: GrayScaleSensor not initialized!");
      return 0;
    }
    
    if (index >= 0 && index < 8) {
      uint8_t recv_value = read();
      return (recv_value >> index) & 1;
    } else {
      Serial.println("Invalid sensor index!");
      return 0;
    }
  }
};

extern GrayScaleSensor gw_linesensor;

class LineSensorArrayNew {
private:

  int* sensorNames;        // 传感器名称数组（此处应该改名为sensorPins表示是数字引脚编号）
  int* sensorThresholds;   // 传感器阈值数组
  float* sensorWeights;    // 传感器权值数组
  int* eepromAddresses;    // EEPROM 地址数组
  float lastLinePosition;  // 存储最后一次检测到线的位置
  float min_value;
  float max_value;
  String color_state;

public:
  LineSensorArrayNew(int num, int* pins, float* weights, int* addresses, String color_state);
  void initMinMaxValue(float min_value_input, float max_value_input);
  int16_t readChannel(int pin);
  void calculateThresholds(int numReadings);
  void loadThresholdsFromEEPROM(int edit_value[]);
  bool getSingleSensor(int i);
  float getLinePosition();
  int* getThresholds();
  void updateSensorWeights(float* newSensorWeights);

  uint8_t numSensors;  // 传感器数量
};

Position decodePosition(const uint8_t* response);
Position ReadChassisDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd);
uint8_t* readI2CMsgChassisDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd); 

void GoSpeedNoPid(int v_x, int v_y, int w);
void GoSpeedWithPid(int v_x, int v_y, int w);
void GoDistance(int x_distance, int y_distance, int angle, int8_t numMotors, int8_t maxSpeed, int8_t acceleration, int8_t deceleration);
bool GetPosiState(uint8_t address, uint8_t motorID);
void InitPosiPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d);
void InitSpeedPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d);
void resetI2Cbus(byte sdaPin, byte sclPin);
void GetIMU(uint8_t address, float& angle_x, float& angle_y, float& angle_z);
void InitIMUAngle(uint8_t address);
void InitStopPIDParam(uint8_t address, uint8_t motorID, float k_p, float k_i, float k_d);
void InitChassisStraightLineAnglePIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d);
uint8_t* createChassisPIDMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int Kp, int Ki, int Kd);
uint8_t* readI2CMsgChassisPID(uint8_t address, uint8_t chassis, uint8_t cmd_chassis);
void PIDSpeedCtl(uint8_t address, uint8_t motorID, int speed_input, int limit = 100);
void PWMSpeedCtl(uint8_t address, uint8_t motorID, int speed_input);
void SoftPWMSpeedCtl(uint8_t address, uint8_t motorID, int speed1, int speed2, int acc);
void DistanceCtl(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance);
void StopCtl(uint8_t address, uint8_t motorID);
int16_t ReadEncoderSpeed(uint8_t address, uint8_t motorID);
int32_t ReadEncoderDistance(uint8_t address, uint8_t motorID);
void zuoQian(int ctrl_speed, int target_speed, int acc);
void zuoHou(int ctrl_speed, int target_speed, int acc);
void youHou(int ctrl_speed, int target_speed, int acc);
void youQian(int ctrl_speed, int target_speed, int acc);
int16_t readChannel(String channelName);
uint8_t* sendI2CMsgPIDSetting(uint8_t address, uint8_t pid_setting);
void PIDSettingOK(uint8_t address, uint8_t pid_setting);
void ChassisGoV(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float w_speed);
void ChassisGoVAngleLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float angle_lock);
uint8_t* createVChassisCtrlMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int v_speed, float v_angle, float angle_lock, float w_speed);
uint8_t* createResetMsg(uint8_t address);
uint8_t* createI2CMsgSetDiameter(uint8_t address, uint8_t motorID, int diameter);
uint8_t* createSpeedPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd);
uint8_t* createGetIMUMsg(uint8_t address);
uint8_t* createInitIMUAngleMsg(uint8_t address);
uint8_t* readI2CMsgSpeedPID(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgCmd(uint8_t address);
bool parseIMUData(byte* RxData, int16_t* angle_x, int16_t* angle_y, int16_t* angle_z);
bool parseData(byte* RxData, int* Kp, int* Ki, int* Kd);
void GetIMUPara(uint8_t address, int& angle_x, int& angle_y, int& angle_z);
void GetPIDPara(uint8_t address, int& Kp, int& Ki, int& Kd);
void GetCMDPara(uint8_t address, byte& cmd);
uint8_t* createPosiPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd);
uint8_t* readI2CMsgPosiPID(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgEncoderSpeed(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgRealtimeSpeed(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgEncoderDistance(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgAbsEncoderDistance(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgDistance(uint8_t address, uint8_t motorID);
uint8_t* readI2CMsgAbsDistance(uint8_t address, uint8_t motorID);
uint8_t* createI2CMsgPwmSpeed(uint8_t address, uint8_t motorID, int speed);
uint8_t* createI2CMsgEncoderSpeed(uint8_t address, uint8_t motorID, int speed);
uint8_t* createDistanceMsg(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance);
uint8_t* createEncoderDistanceMsg(uint8_t address, uint8_t motorID, uint8_t initialSpeed, uint8_t maxSpeed, uint8_t acceleration, uint8_t deceleration, int16_t totalDistance);
uint8_t* readI2CMsgHaveGotTargetPosi(uint8_t address, uint8_t motorID);
uint8_t* createI2CSpeedControlMsg(uint8_t address, uint8_t motorID, int speed1, int speed2, int acc);
uint8_t* createI2CMsgSetCircleCnt(uint8_t address, uint8_t motorID, int circle_cnt);
uint8_t* createStopPIDMsg(uint8_t address, uint8_t motorID, int Kp, int Ki, int Kd);
uint8_t* readI2CMsgStopPID(uint8_t address, uint8_t motorID);
uint8_t* sendI2CMsgStop(uint8_t address, uint8_t motorID);
byte computeXOR(uint8_t* data, int length);
int32_t GetDistance(uint8_t address);
int16_t GetSpeed(uint8_t address);
void ChassisTurnAngleWoLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float v_angle, String turn_state, float w_speed, GrayScaleSensor sensor) ;
void ChassisRelease(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle, String turn_state, float speed_rate, GrayScaleSensor sensor);
uint8_t* createChassisTurnAngleWoLockMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, int turn_state, float v_angle, float w_speed);
void ChassisGoDistance(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float angle, float angle_lock, int distance, int speed_rate, int init_speed = 3);

uint8_t* createChassisGoDistanceMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float angle, float angle_lock, int distance, int speed_rate, int init_speed);
void InitChassisPositionCorrectionPIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d);
void InitChassisTurnAngleLockPIDParam(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float k_p, float k_i, float k_d);
uint8_t* createChassisStopMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd);
void ChassisStop(uint8_t address, uint8_t chassis, uint8_t chassis_cmd);
void ChassisAngleLock(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle);
uint8_t* createChassisAngleLockMsg(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float target_angle);

float getZAngle();

class Parameter {
public:
  Parameter(const String& displayName, int eepromAddr, void (*function)(), bool isFloat = false);

  void increase();
  void decrease();
  float getValue();
  String getName();
  void execute();
  bool isFloatValue();

private:
  String name;
  uint8_t addr;
  float value = 0;
  bool isFloat;
  void (*executeFunction)();
};

class PID {
public:
  PID(float kp, float ki, float kd, float minInte, float maxInte);
  int calcPid(float input);

private:
  float Kp;
  float Ki;
  float Kd;
  float erroInte;
  float errorLast;
  float minErroInte;
  float maxErroInte;
};


void lineFollowPIDDistanceSpeedDual(int32_t distance, int target_speed, float acceleration = 100);
void lineFollowPIDSideSpeedDual(String side_choice, int target_speed, float acceleration = 100);
void turnToAngleDual(float target_angle, String turn_state, float speed_rate, GrayScaleSensor sensor = gw_linesensor);
void ChassisGoDistanceDual(int lock_angle, int distance, int speed);
void stopDual();
void ChassisGoSpeed(uint8_t address, uint8_t chassis, uint8_t chassis_cmd, float angle, float angle_lock, int distance, int speed_rate, int init_speed = 3);
void ChassisGoSpeedDual(int lock_angle, int speed, String side_choice);
void releaseDual(GrayScaleSensor sensor);
void angleLockDual(float lock_angle);
void ZuoLunSuDu(int current_speed);
void YouLunSuDu(int current_speed);

#if defined(ESP_PLATFORM)
/* ESP32 */


#endif

#endif /* INC_GW_GRAYSCALE_SENSOR_H_ */
