#include "DeDeEncCarBase_V1_0.h"

MyNeoPixel myStrip(NUM_LEDS, PIN);

// 使用硬件I2C接口初始化显示屏
U8X8_SSD1306_128X32_UNIVISION_HW_I2C u8x8(/* reset=*/U8X8_PIN_NONE);

// INA219的初始化
Adafruit_INA219 ina219;

// 使用默认构造函数创建GrayScaleSensor对象
GrayScaleSensor gw_linesensor;

// 用于存储调整值的EEPROM地址
const int EEPROM_ADDRESS_LEFT_HAND = 2000;

const int EEPROM_ADDRESSES[3] = { 2000, 2000 + sizeof(int), 2000 + sizeof(int) * 2 };

// PID的初始化
// 循迹的PID参数
PID xunji(10, 0.0, 25, -8, 8);

// 定义D4为RX, D5为TX
SoftwareSerial mySerial(8, 4);  // RX, TX
SimpleServoControl servoControl(mySerial);

// 定义软件串口的RX和TX引脚
const int rxPin = 3;  // 修改为你的接收引脚
const int txPin = 2;  // 修改为你的发送引脚

// 接收传感器
int jiance_wukuai = 11;

// 模式选择全局变量
int selectedMode = 0; // 0: 模式1, 1: 模式2, 2: 模式3

void setup() {
  pinMode(jiance_wukuai, INPUT);
  // put your setup code here, to run once:
  Wire.begin();          // 加入 I2C 总线
  Serial.begin(115200);  // 初始化串口通信，波特率设为9600

  gw_linesensor.begin(GW_GRAY_ADDR);

  mySerial.begin(19200);

  myStrip.begin();

  delay(500);

  // 蜂鸣器鸣叫
  myStrip.red();
  delay(500);
  myStrip.turnOff();
  delay(500);
  myStrip.red();
  delay(500);
  myStrip.turnOff();
  delay(500);
  myStrip.red();
  delay(500);
  myStrip.turnOff();
  delay(500);
  myStrip.blue();

  // 按下按钮初始化陀螺仪标定
  while (1) {
    if (analogRead(A3) <= 100) {
      delay(10);
      if (analogRead(A3) <= 100) {
        break;
      }
    }
  }

  InitIMUAngle(STM32_ADDRESS_0);  // 标定陀螺仪

  myStrip.green();

  Wire.begin();  // 加入 I2C 总线 主要是和I2C通讯
  InitIMUAngle(STM32_ADDRESS_0);
  delay(100);
  InitIMUAngle(STM32_ADDRESS_0);
  delay(200);

  // 初始化INA219
  if (!ina219.begin()) {
    Serial.println("Failed to find INA219 chip");
    while (1) {
      delay(10);
    }
  }

  float voltage_sum = 0;

  for (int i = 0; i < 10; i++) {
    float vol_now = ina219.getBusVoltage_V();
    voltage_sum += vol_now;
  }

  float real_voltage = voltage_sum / 10;
  Serial.print("voltage = ");
  Serial.println(real_voltage);

  if (real_voltage <= 7.6) {
    myStrip.white();
    delay(500);
    myStrip.turnOff();
    delay(500);
    myStrip.white();
    delay(500);
    myStrip.turnOff();
    delay(500);
    myStrip.white();
    delay(500);
    myStrip.turnOff();
    delay(500);
    myStrip.white();
  }


  // PID的设定
  InitSpeedPIDParam(STM32_ADDRESS_0, MOTOR_ID_0, 70, 275, 0.2);
  InitSpeedPIDParam(STM32_ADDRESS_0, MOTOR_ID_1, 70, 275, 0.2);
  InitSpeedPIDParam(STM32_ADDRESS_0, MOTOR_ID_2, 70, 275, 0.2);
  InitSpeedPIDParam(STM32_ADDRESS_0, MOTOR_ID_3, 70, 275, 0.2);

  // 初始化梯形位置控制PID
  InitPosiPIDParam(STM32_ADDRESS_0, MOTOR_ID_0, 3, 2, 0);
  InitPosiPIDParam(STM32_ADDRESS_0, MOTOR_ID_1, 3, 2, 0);
  InitPosiPIDParam(STM32_ADDRESS_0, MOTOR_ID_2, 3, 2, 0);
  InitPosiPIDParam(STM32_ADDRESS_0, MOTOR_ID_3, 3, 2, 0);

  // 初始化刹车的位置PID
  InitStopPIDParam(STM32_ADDRESS_0, MOTOR_ID_0, 17, 8, 0.4);
  InitStopPIDParam(STM32_ADDRESS_0, MOTOR_ID_1, 17, 8, 0.4);
  InitStopPIDParam(STM32_ADDRESS_0, MOTOR_ID_2, 17, 8, 0.4);
  InitStopPIDParam(STM32_ADDRESS_0, MOTOR_ID_3, 17, 8, 0.4);

  // 设定直走保持角度的PID
  //
  // InitChassisStraightLineAnglePIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_STRAIGHT_LINE_ANGLE, 1.6, 0.0, 16);
  InitChassisPositionCorrectionPIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_Position_Correction, 10, 0.18, 8);
  // InitChassisTurnAngleLockPIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_Turn_Angle_Lock, 1.6, 0.05, 16);

  InitChassisStraightLineAnglePIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_STRAIGHT_LINE_ANGLE, 0.8, 0.0, 8);
  // InitChassisPositionCorrectionPIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_Position_Correction, 0, 0, 0);
  InitChassisTurnAngleLockPIDParam(STM32_ADDRESS_0, CHASSIS, CHASSIS_Turn_Angle_Lock, 1, 0.05, 8);


  // while(1);

  // PID设定完成
  PIDSettingOK(STM32_ADDRESS_0, PID_STATE);
  delay(1000);

  Serial.println("finish PID setting!");

  // 设定那个循迹板子
  float gwSensorWeights[] = { 0.8, 0.6, 0.4, 0.2, -0.2, -0.4, -0.6, -0.8 };
  gw_linesensor.setWeights(gwSensorWeights);

  Wire.begin();  // 加入 I2C 总线 主要是和I2C通讯

  // 初始化OLED显示
  u8x8.begin();
  u8x8.setFont(u8x8_font_chroma48medium8_r);

  Serial.println("finish u8x8 setting!");

  // 所有状态初始化完成
  u8x8.clear();  // 清除屏幕

  myStrip.green_blue();

  // // 选择模式
  // // 有三个按钮
  // // 没有按下是1023
  // // 中间按下小于10
  // // 左边切换 < 900
  // // 右边切换 < 600

  // u8x8.clear();
  // u8x8.setCursor(0, 0);
  // u8x8.print("选择模式:");

  // while(1) {
  //   // 显示当前模式
  //   u8x8.setCursor(0, 1);
  //   u8x8.print("模式 ");
  //   u8x8.print(selectedMode + 1);
  //   u8x8.print("    ");
    
  //   // 显示对应颜色
  //   switch(selectedMode) {
  //     case 0:
  //       myStrip.red();
  //       break;
  //     case 1:
  //       myStrip.green();
  //       break;
  //     case 2:
  //       myStrip.blue();
  //       break;
  //   }
    
  //   int buttonValue = analogRead(A3);
  //   Serial.println(buttonValue);
    
  //   // 左边按钮按下 - 减少模式
  //   if (buttonValue < 900 && buttonValue >= 600) {
  //     delay(10); // 消抖
  //     if (analogRead(A3) < 900 && analogRead(A3) >= 600) {
  //       selectedMode = (selectedMode > 0) ? selectedMode - 1 : 2; // 减少并循环
  //       delay(300); // 防止快速切换
  //     }
  //   }
    
  //   // 右边按钮按下 - 增加模式
  //   if (buttonValue < 600 && buttonValue >= 10) {
  //     delay(10); // 消抖
  //     if (analogRead(A3) < 600 && analogRead(A3) >= 10) {
  //       selectedMode = (selectedMode + 1) % 3; // 增加并循环
  //       delay(300); // 防止快速切换
  //     }
  //   }
    
  //   // 中间按钮按下 - 确认选择
  //   if (buttonValue < 10) {
  //     delay(10); // 消抖
  //     if (analogRead(A3) < 10) {
  //       // 双闪当前选择的颜色灯
  //       myStrip.turnOff();
  //       delay(300);
  //       // 第一闪
  //       switch(selectedMode) {
  //         case 0: myStrip.red(); break;
  //         case 1: myStrip.green(); break;
  //         case 2: myStrip.blue(); break;
  //       }
  //       delay(300);
  //       myStrip.turnOff();
  //       delay(300);
  //       // 第二闪
  //       switch(selectedMode) {
  //         case 0: myStrip.red(); break;
  //         case 1: myStrip.green(); break;
  //         case 2: myStrip.blue(); break;
  //       }
  //       delay(300);
  //       myStrip.turnOff();
        
  //       break; // 退出循环
  //     }
  //   }
    
  //   delay(100);
  // }

  // // 确认选择并继续
  // Serial.print("已选择模式: ");
  // Serial.println(selectedMode + 1);

  myStrip.green_blue();
  // 按钮再按下一次
  while (1) {
    if (analogRead(A3) <= 100) {
      delay(10);
      if (analogRead(A3) <= 100) {
        break;
      }
    }
  }

  Wire.begin();  // 加入 I2C 总线 主要是和I2C通讯
}

/* 舵机的控制 */
// 安装初始化
void AnZhuang(){
  servoControl.set_angle(0, 90, 50);  // 40是夹紧
  delay(10);

  servoControl.set_angle(1, 90, 50);  // 40是夹紧
  delay(10);

  servoControl.set_angle(2, 90, 50);  // 40是夹紧
  delay(10);

}

// 机械臂放下
void FangXia(){
  servoControl.set_angle(0, 90, 50);  // 40是夹紧
  delay(10);

  servoControl.set_angle(1, 90, 50);  // 40是夹紧
  delay(10);

}

// 机械臂抬升
void TaiSheng(){
  servoControl.set_angle(0, 20, 50);  // 40是夹紧
  delay(10);

  servoControl.set_angle(1, 20, 50);  // 40是夹紧
  delay(10);
}

// 爪子打开
void ZhuaZiDaKai(){
  servoControl.set_angle(2, 130, 50);  // 40是夹紧
  delay(10);
}

// 爪子抓大圆柱
void ZhuaZiDaYuan(){
  servoControl.set_angle(2, 80, 50);  // 40是夹紧
  delay(10);
}

// 爪子抓小圆柱
void ZhuaZiXiaoYuan(){
  servoControl.set_angle(2, 80, 50);  // 40是夹紧
  delay(10);
}

// 爪子抓球
void ZhuaZiQiu(){
  servoControl.set_angle(2, 80, 50);  // 40是夹紧
  delay(10);
}

/* 底盘的控制 */
// 速度距离循迹
void SuDuJuLiXunJi(int sudu, int juli, int jiasudu){
  lineFollowPIDDistanceSpeedDual(juli, sudu, jiasudu);
}

// 循迹打到黑边
// 循迹打到
void XunJiDaDao(String weizhi, int sudu){
  lineFollowPIDSideSpeedDual(weizhi, sudu);
}

// 陀螺仪锁定角度走距离
// 陀螺仪锁走距离
void TuoLuoYiSuoZouJuLi(int jiaodu, int juli, int jiasudu){
  ChassisGoDistanceDual(jiaodu, juli, jiasudu);
}

// 刹车
void ShaChe(){
  stopDual();
}

// 转到某个角度
void ZhuanDaoJiaoDu(int jiaodu, String fangxiang, int sudu){
  turnToAngleDual(jiaodu, fangxiang, sudu);
}

void loop() {
  TaiSheng();
  ZhuaZiDaKai();
  SuDuJuLiXunJi(1900, 10, 1);
  XunJiDaDao("both_sid", 10);

  TuoLuoYiSuoZouJuLi(180, 115, 10);
  ZhuanDaoJiaoDu(-90, "shun", 10);

  SuDuJuLiXunJi(200, 10, 1);
  XunJiDaDao("mid", 10);

  SuDuJuLiXunJi(350, 10, 1);
  ShaChe();
  delay(300);

  TuoLuoYiSuoZouJuLi(-90, -220, 10);

  ZhuanDaoJiaoDu(0, "shun", 10);

  SuDuJuLiXunJi(200, 10, 1);
  XunJiDaDao("mid", 10);

  TuoLuoYiSuoZouJuLi(0, 115, 10);
  ZhuanDaoJiaoDu(-80, "ni", 10);

  SuDuJuLiXunJi(400, 8, 1);
  XunJiDaDao("you", 10);

  TuoLuoYiSuoZouJuLi(-90, 115, 10);

  ZhuanDaoJiaoDu(180, "ni", 10);

  SuDuJuLiXunJi(200, 10, 1);

  TuoLuoYiSuoZouJuLi(180, -130, 10);

  ZhuanDaoJiaoDu(0, "ni", 10);

  // 准备去抓取平台
  SuDuJuLiXunJi(400, 10, 1);
  XunJiDaDao("you", 10);

  FangXia();

  SuDuJuLiXunJi(370, 8, 1);
  XunJiDaDao("you", 8);

  ShaChe();
  ZhuaZiDaYuan();
  delay(500);
  TaiSheng();
  ZhuanDaoJiaoDu(-18, "ni", 10);

  TuoLuoYiSuoZouJuLi(-18, 190, 7);

  ZhuaZiDaKai();
  delay(500);
  TuoLuoYiSuoZouJuLi(-18, -60, 7);

  ZhuanDaoJiaoDu(90, "shun", 10);

  // 循迹乒乓球

  SuDuJuLiXunJi(1500, 10, 1);
  XunJiDaDao("you", 10);

  FangXia();

  SuDuJuLiXunJi(400, 5, 1);
  XunJiDaDao("you", 5);

  ShaChe();
  ZhuaZiQiu();

  delay(500);
  TaiSheng();
  ZhuanDaoJiaoDu(72, "ni", 10);

  TuoLuoYiSuoZouJuLi(72, 190, 7);

  ZhuaZiDaKai();
  delay(500);
  TuoLuoYiSuoZouJuLi(72, -60, 7);

  ZhuanDaoJiaoDu(180, "shun", 10);

  SuDuJuLiXunJi(800, 10, 1);
  XunJiDaDao("you", 10);
  TuoLuoYiSuoZouJuLi(180, 115, 10);

  ZhuanDaoJiaoDu(-90, "shun", 10);

  SuDuJuLiXunJi(1150, 7, 1);
  XunJiDaDao("zuo", 7);
  TuoLuoYiSuoZouJuLi(180, 115, 10);

  ZhuanDaoJiaoDu(90, "ni", 10);
  FangXia();
  ZhuaZiDaKai();

  SuDuJuLiXunJi(200, 5, 1);
  XunJiDaDao("you", 5);

  ShaChe();

  ZhuaZiXiaoYuan();

  delay(500);

  ZhuanDaoJiaoDu(60, "ni", 10);
  ZhuaZiDaKai();
  delay(500);
  TaiSheng();
  delay(500);
  ZhuanDaoJiaoDu(90, "shun", 10);

  TuoLuoYiSuoZouJuLi(90, -900, 14);

  while (1)
    ;
}